rm(list=ls())
gc(TRUE)

##====================== Initialisation ========================####
if (TRUE){
  setwd("C:/GoogleDrive/MyDocs/Annastiina/Topics/MTVGJR_MGARCH/Project2_Building")        # Anna's work PC  - GOOGLE DRIVE
  #setwd("~/Project2_Building")                                         
  
  source("clsTV.r")
  library(parallel)
  library(RevoUtilsMath)
  maxThreads <- detectCores()-1
  setMKLthreads(maxThreads)
}
##====================== Initialisation ========================##


##====================== Data Load =============================####
if (TRUE){
  # Read AUS_4Banks data from saved file
  mydata <- readRDS("AUS_bankreturns_1992_2020.RDS")
  dates <- mydata$date
  
  # Create scaled-up & de-meaned individual data series vectors (Percentage Returns):
  e_cba <- mydata$return_cba * 100 
  e_cba <- e_cba - mean(e_cba)
  e <- e_cba

  # Plot the returns
  ptitle <- "CBA Returns"
  plot(dates,e_cba,type="l",main=ptitle)
  
  # Read in the Garch'd Reference Data
  RefData <- readRDS("Gen_Refdata/RefData_withGarch_CBA_2020.RDS")
  
  # Tidy Up
  rm(mydata)
  gc(TRUE)
}
##====================== Data Load =============================##


##   
##============  General Specification Method    ===============####
##
##   1. Plot the local Variance and visually observe any transitions
##   2. Calculate a set of *indicative* garch parameters using different window lengths & kurtosis
##   3. Identify the first "single" transition start & end:
##     4. Generate refData0 using TV$delta0 = var(e[start:end])
##     5. Test for a first transition in this subset of the time series 
##     6. Generate refData1 using TV$delta0 & TV$pars = TV estimates found above
##     7. Test for a second transition in this subset of the time series, and if none is found, move on to the next subset 
##   8. Repeat steps 4 - 7 for each subsequent data subset where we think there may be a transition
##   9. Complete the specification by adding the individual sub-sections together, re-estimating & re-testing
##  10. Consider if any of the transitions could be re-parameterised as a double-transition to reduce the total number of parameters
##  11. Re-estimate the final model and calculate & check the standard errors

##==============================================================##

# Plot returns using index as x-axis:
ptitle <- "CBA Returns"
plot(e,type="l",main = ptitle)
abline(h=seq(-010,010,5), v=seq(0,7000,500),col="lightgrey")
lines(e)


##==============================================================##
##  TV.ORDER 0, obs 1:7108  ####
##==============================================================##

if (TRUE) {

  e <- e_cba
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs

  TV <- tv(st,tvshape$delta0only)
  TV <- estimateTV(e,TV)

  # Now Test against CBA
  TV <- setTaylorOrder(1,TV)
  
  RefTests2 <- list()
  RefTests2$LMTR2 <- LM.TR2(e,TV)
  RefTests2$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "CBA.ORD0.1_1-7108.RDS"
  simcontrol$numLoops <- 100
  simcontrol$numCores <- 3
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)

  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}  
##==============================================================##
##  TV - AUS_4Banks/CBA ---- ORDER 0.1 ---- obs 1:7108          ##
## Execution time: 775 sec on Glens Work PC
## Result: Pvals=[TR2:58%,Robust=38%] 
## Fail to Reject the null: H0=Model is sufficient
##  This is likely caused by issues with the optimiser 
##  where there are many transitions
##  So, try a smaller range... 
##==============================================================##



##==============================================================##
##  TV.ORDER 0, obs 1:4000  ####
##==============================================================##

if (TRUE) {
  
  e <- e_cba[1:4000]
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  TV <- tv(st,tvshape$delta0only)
  TV <- estimateTV(e,TV)
  
  # Now Test against CBA
  TV <- setTaylorOrder(3,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "CBA.ORD0.3_1-4000.RDS"
  simcontrol$numLoops <- 1200
  simcontrol$numCores <- 3
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)
  
  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}  # End: if (DoThis)...
##==============================================================##
##  TV - AUS_4Banks/CBA ---- obs 1:4000          
## Execution time: 235 sec on Glens Work PC
## ORDER 0.1 
## Result: Pvals=[TR2=79%,Robust=77%] 
## ORDER 0.2
## Result: Pvals=[TR2=21%,Robust=31%] 
## ORDER 0.3
## Result: Pvals=[TR2=36%,Robust=40%] 

## Fail to Reject the null: H0=Model is sufficient
## Conclusion: No evidence of a transition here

##==============================================================##



##==============================================================##
##  TV.ORDER 0, obs 1:5000  ####
##==============================================================##

if (TRUE) {
  
  e <- e_cba[1:5000]
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  TV <- tv(st,tvshape$delta0only)
  TV <- estimateTV(e,TV)
  
  # Now Test against CBA
  TV <- setTaylorOrder(2,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "CBA.ORD0.2_1-5000.RDS"
  simcontrol$numLoops <- 1200
  simcontrol$numCores <- 3
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)
  
  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}  # End: if (DoThis)...
##==============================================================##
##  TV - AUS_4Banks/CBA ---- ---- obs 1:5000          
## Execution time: 235 sec on Glens Work PC
## ORDER 0.1 
## Result: Pvals=[TR2=0.9%,Robust=1%] 
## ORDER 0.2
## Result: Pvals=[TR2=2.4%,Robust=3.8%] 
## ORDER 0.3
## Result: Pvals=[TR2=%,Robust=%] 

## Reject the null
## Conclusion: Strong evidence of a linear transition here

##==============================================================##



##==============================================================##
##  TV.ORDER 1, obs 1:5000  ####
##==============================================================##

if (TRUE) {
  
  e <- e_cba[1:5000]
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  TV <- tv(st,tvshape$single)
  
  # Add some estimation controls
  estCtrl <- list()
  estCtrl$calcSE <- TRUE
  estCtrl$verbose <- TRUE
  
  OptCtrl <- TV$optimcontrol
  OptCtrl$reltol <- 1e-9
  OptCtrl$ndeps <- OptCtrl$ndeps * 1e-2
  #OptCtrl$parscale
  TV$optimcontrol <- OptCtrl
  
  TV$delta0
  TV$pars[1,1] <- 0.05
  TV$pars[2,1] <- 2.05
  TV$pars[3,1] <- 0.65

  TV <- estimateTV(e,TV,estCtrl)
  
  summary(TV)
  plot(TV)
  
  # Now Test against CBA
  TV <- setTaylorOrder(1,TV)
  
  plot(TV@st)
  
  RefTests1 <- list()
  RefTests1$LMTR2 <- LM.TR2(e,TV)
  RefTests1$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "CBA.ORD1.2_1-5000.RDS"
  simcontrol$numLoops <- 1200
  simcontrol$numCores <- 3
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)
  
  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}  # End: if (DoThis)...
##==============================================================##
##  TV - AUS_4Banks/CBA ---- ---- obs 1:5000          
## Execution time: 235 sec on Glens Work PC
## ORDER 0.1 
## Result: Pvals=[TR2=0.9%,Robust=1%] 
## ORDER 0.2
## Result: Pvals=[TR2=2.4%,Robust=3.8%] 
## ORDER 0.3
## Result: Pvals=[TR2=%,Robust=%] 

## Reject the null
## Conclusion: Strong evidence of a linear transition here

##==============================================================##

# Estimation Results:
#   
#   Delta0 = 0.000119 se0 =  0 
# 
# st1     se1
# deltaN 0.000326 2.1e-05
# speedN 7.000000     NaN
# locN1  0.811560 7.7e-04
# locN2        NA     NaN
# 
# Log-liklihood value:  14870.11

##==============================================================##

##  TV - AUS_4Banks/CBA ---- ---- obs 1:5000          
## Execution time:  sec on Glens Work PC
## ORDER 1.1 
## Result: Pvals=[TR2= 50%,Robust= 71%] 
## ORDER 1.2
## Result: Pvals=[TR2= 11%,Robust= 17%] 
## ORDER 1.3
## Result: Pvals=[TR2= %,Robust=%] 

## Cannot Reject the null
## Conclusion: No evidence of another transition here

##==============================================================##



##==============================================================##
##  TV.ORDER 0, obs 5001:7108  ####
##==============================================================##

if (TRUE) {
  
  e <- e_cba[5001:7108]
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  TV <- tv(st,tvshape$delta0only)
  
  # Add some estimation controls
  estCtrl <- list()
  estCtrl$calcSE <- TRUE
  estCtrl$verbose <- TRUE
  
  OptCtrl <- TV$optimcontrol
  OptCtrl$reltol <- 1e-11
  OptCtrl$ndeps <- OptCtrl$ndeps * 1e-2
  #OptCtrl$parscale
  TV$optimcontrol <- OptCtrl
  
  TV$delta0
  TV$pars[1,1] <- 0.05
  TV$pars[2,1] <- 2.05
  TV$pars[3,1] <- 0.65
  
  TV <- estimateTV(e,TV,estCtrl)
  
  summary(TV)
  plot(TV)
  
  # Now Test against CBA
  TV <- setTaylorOrder(3,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "CBA.ORD0.3_5001-7108.RDS"
  simcontrol$numLoops <- 1200
  simcontrol$numCores <- 3
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)
  
  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}  # End: if (DoThis)...
##==============================================================##
##  TV - AUS_4Banks/CBA ---- ---- obs 5001:7108
## ORDER 0.1 
## Result: Pvals=[TR2= 93%,Robust= 93%] 
## ORDER 0.2
## Result: Pvals=[TR2= 84%,Robust= 90%] 
## ORDER 0.3
## Result: Pvals=[TR2= 84%,Robust= 73%] 

## Fail to Reject the null
## Conclusion: No evidence of a transition here

##==============================================================##






##==============================================================##
##  TV.ORDER 1, obs 1:7108  ####
##==============================================================##

if (TRUE) {
  
  e <- e_cba
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  TV <- tv(st,tvshape$single)
  
  # Add some estimation controls
  estCtrl <- list()
  estCtrl$calcSE <- TRUE
  estCtrl$verbose <- TRUE
  
  OptCtrl <- TV$optimcontrol
  OptCtrl$reltol <- 1e-11
  OptCtrl$ndeps <- OptCtrl$ndeps * 1e-2
  #OptCtrl$parscale
  TV$optimcontrol <- OptCtrl
  
  TV$delta0
  TV$pars["deltaN",1] <- 0.05
  TV$pars["speedN",1] <- 2.05
  TV$pars["locN1",1] <- 0.65 * 5/7
  
  TV <- estimateTV(e,TV,estCtrl)
  
  summary(TV)
  plot(TV)
  
  # Now Test against CBA
  TV <- setTaylorOrder(2,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "CBA.ORD1.2_1-7108.RDS"
  simcontrol$numLoops <- 1200
  simcontrol$numCores <- 3
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)
  
  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}

##==============================================================##

# Estimation Results:
#   
#   Delta0 = 0.000118 se0 =  3e-06 
# 
#           st1      se1
# deltaN 0.000094 0.000006
# speedN 7.000000 0.000000
# locN1  0.554429 0.001029
# locN2        NA      NaN
# 
# Log-liklihood value:  21136.26
##==============================================================##

##==============================================================##
##  TV - AUS_4Banks/CBA ---- ---- obs 1:7108          
## ORDER 1.1 
## Result: Pvals=[TR2= 9.85%,Robust= 3.35%] 
## ORDER 1.2
## Result: Pvals=[TR2= 0.93%,Robust= 1.5%] 
## ORDER 1.3
## Result: Pvals=[TR2= %,Robust= %] 

## Reject the null
## Conclusion: Evidence of a double transition here

##==============================================================##


##==============================================================##
##  TV.ORDER 2, obs 1:7108  ####
##==============================================================##

if (TRUE) {
  
  e <- e_cba
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  # Add some estimation controls
  estCtrl <- list()
  estCtrl$calcSE <- TRUE
  estCtrl$verbose <- TRUE
  
  TV <- tv(st,c(tvshape$single,tvshape$double1loc))

  OptCtrl <- TV$optimcontrol
  OptCtrl$reltol <- 1e-9
  OptCtrl$ndeps <- c(1e-7,1e-7,1e-7,1e-5,1e-7,1e-7,1e-5)
  OptCtrl$parscale <- c(6,6,2,1,6,2,1)
  TV$optimcontrol <- OptCtrl
  
  TV$delta0 <- 1
  TV$pars["deltaN",] <- c(1,1)
  TV$pars["speedN",1] <- 4.05
  TV$pars["locN1",1] <- 0.5
  TV$pars["speedN",2] <- 2.05
  TV$pars["locN1",2] <- 0.6
  
  TV <- estimateTV(e,TV,estCtrl)
  
  summary(TV)
  plot(TV)
  
  # Now Test against CBA
  TV <- setTaylorOrder(2,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "CBA.ORD2sd.1_1-7108.RDS"
  simcontrol$numLoops <- 1200
  simcontrol$numCores <- 3
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)
  
  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}

##==============================================================##

# TV OBJECT
# 
# Estimation Results:
#   
#   Delta0 = 14.58511 se0 =  0.886324 
# 
#                st1      se1 sig        st2      se2   sig.1
#   deltaN -0.356224 0.043148 *   -13.238601 0.885749   *  
#   speedN  6.987267 1.762909 *     7.000000 0.000000   ***
#   locN1   0.404079 0.002251 **    0.603709 0.001471   ***
#   locN2         NA      NaN             NA      NaN      
# 
# Log-liklihood value:  -11095.8
##==============================================================##

##==============================================================##
##  TV - AUS_4Banks/CBA ---- ---- obs 1:7108          
## ORDER 2.1 
## Result: Pvals=[TR2= 17%,Robust= 21%] 
## ORDER 2.2
## Result: Pvals=[TR2= 16%,Robust= 20%] 

## Fail to Reject the null
## Conclusion: No Evidence of another transition here
##             Model is fully specified
##==============================================================##


##==============================================================##
##  TV.ORDER 2, MTVGJR Model  ####
##==============================================================##

##====================== Data Load =============================####
if (TRUE){
  # Read AUS_4Banks data from saved file
  mydata <- readRDS("AUS_bankreturns_1992_2020.RDS")
  dates <- mydata$date
  
  # Create scaled-up & de-meaned individual data series vectors (Percentage Returns):
  e_cba <- mydata$return_cba * 100 
  e_cba <- e_cba - mean(e_cba)
  e <- e_cba
  
  # Plot the returns
  ptitle <- "CBA Returns"
  plot(dates,e,type="l",main=ptitle)
  
  # Read in the Garch'd Reference Data
  RefData <- readRDS("Gen_Refdata/RefData_withGarch_CBA_2020.RDS")
  
  # Tidy Up
  rm(mydata)
  gc(TRUE)
}

if (TRUE) {
  
  source("clsMTVGARCH.r")
  
  e <- e_cba
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs

  TV <- tv(st,c(tvshape$single,tvshape$double1loc))
  
  OptCtrl <- TV$optimcontrol
  OptCtrl$reltol <- 1e-9
  OptCtrl$ndeps <- c(1e-5,1e-7,1e-5,1e-7,1e-5,1e-7,1e-7)
  OptCtrl$parscale <- c(6,3,1,1,3,1,1)
  #OptCtrl$parscale <- c(3,3,1,1,3,1,1)
  TV$optimcontrol <- OptCtrl
  
  TV$delta0 <- 1
  TV$pars["deltaN",] <- c(1,1)
  TV$pars["speedN",1] <- 3.05
  TV$pars["locN1",1] <- 0.4
  
  TV$pars["speedN",2] <- 3.05
  TV$pars["locN1",2] <- 0.6
  
  estCtrl <- list(calcSE = TRUE,verbose = TRUE)
  TV <- estimateTV(e,TV,estCtrl)
  
  summary(TV)
  plot(TV)
  
  # Create the MTV-GJR object
  MTV_cba <- mtvgarch(TV,garchType = garchtype$gjr)
  
  # Estimate the MTV-GJR object
  MTV_cba <- estimateMTVGARCH(e,MTV_cba)  
  
  # Look at the univariate MTV-GJR object:
  # Plot univar TV 'g'
  plot(MTV_cba$tv)
  # Overlay the final estimated TV 'g'
  lines(MTV_cba$Estimated$tv@g,type="l",col="red")
  # If the scale is off, plot the Estimated TV 'g'
  plot(MTV_cba$Estimated$tv@g,type="l",col="red")
  
  # Plot univar Garch 'h'
  plot(MTV_cba$garch@h,type="l")
  # Overlay the final estimated Garch 'h'
  lines(MTV_cba$Estimated$garch@h,type="l",col="red")
  # If the scale is off, plot the Estimated Garch 'h'
  plot(MTV_cba$Estimated$garch@h,type="l",col="red")
  
  # Look at the final parameters
  summary(MTV_cba$Estimated$tv)
  summary(MTV_cba$Estimated$garch)
  
  # Alternate look method...
  MTV_cba$Estimated$tv$Estimated$delta0
  MTV_cba$Estimated$tv$Estimated$delta0_se
  MTV_cba$Estimated$tv$Estimated$pars
  MTV_cba$Estimated$tv$Estimated$se
  
  # Save the estimated model
  saveRDS(MTV_cba,"Output/CBA_mtvgjr.RDS")
  
}

#  Univar TV  Estimation Results:
#   
#   Delta0 = 0 se0 =  NaN 
# 
#                st1      se1 sig      st2      se2 sig.1
#   deltaN 1.2438307 0.060485 **  1.242096 0.030272   ** 
#   speedN 4.6148744 0.130642 **  3.097853 0.167118   *  
#   locN1  0.5580971 0.002868 **  0.893777 0.017572   ** 
#   locN2         NA      NaN           NA      NaN      
# 
# Log-liklihood value:  -11512.76



##==============================================================##
##  TV.ORDER 2, MTVGJR Model - Manual Estimation   ####
##==============================================================##

if (TRUE) {
  
  source("clsGARCH.r")
  source("clsMTVGARCH.r")
  
  # Create e, TV1 & G1 
  # make an original copy of the data:
  e_orig <- e
  estCtrl <- list(calcSE = TRUE, verbose = TRUE)
  
  TV1 <- TV  # Estimated above
  
  G1 <- garch(garchtype$gjr)
  
  OptCtrl <- G1$optimcontrol
  OptCtrl$reltol <- 1e-7
  OptCtrl$ndeps <- c(1e-5,1e-5,1e-5,1e-5)
  OptCtrl$parscale <- c(1,1,9,1)
  G1$optimcontrol <- OptCtrl
  
  G1$pars["omega",1] <- 0.01
  G1$pars["alpha",1] <- 0.02
  G1$pars["beta",1] <- 0.8
  G1$pars["gamma",1] <- 0.03
  
  G1 <- estimateGARCH(e,G1,estCtrl)
  summary(G1)
  plot(G1)
  
  MTV1 <- mtvgarch(e,TV1,G1)
  
  ###  START THE PAIR_WISE ESTIMATION HERE  ###
  
  # Filter out Garch & estimate TV component
  if(TRUE) {
    
    #lG1 <- MTV1@garch                # For first iteration
    lG1 <- MTV1$results[[2]]$garch   # For subsequent iterations - update the index!
    e <- e_orig/sqrt(lG1@h)
    
    #lT1 <- MTV1@tv                # For first iteration
    lT1 <- MTV1$results[[2]]$tv   # For subsequent iterations - update the index!
    
    summary(lT1)
    ll.target <- getTargetValue(e,lT1)
    
    lT1$delta0 <- lT1$Estimated$delta0
    lT1$pars <- lT1$Estimated$pars 
    
    lT1$pars["deltaN",] <- c(1.0,1.0)
    lT1$pars["speedN",] <- c(4.0,6.0)
    lT1$pars["locN1",] <- c(0.1,0.5)
    #lT1$pars["locN2",] <- 0
    
    optCtrl <- lT1$optimcontrol
    optCtrl$reltol <- 1e-7
    optCtrl$ndeps <- c(1e-7,1e-5,1e-7,1e-7,1e-5,1e-7)
    #optCtrl$ndeps <- c(1e-9,1e-7,1e-7,1e-9,1e-7,1e-7)
    optCtrl$parscale <- c(3,1,1,3,1,1)
    #optCtrl$parscale <- c(6,2,1,7,2,1)
    lT1$optimcontrol <- optCtrl
    
    lT1 <- estimateTV(e,lT1,estCtrl)
    summary(lT1)  
    plot(lT1)
    
    # Check that the LL is higher than the target:
    lT1$Estimated$value > ll.target 
    
  }
  
  # Filter out TV & estimate GARCH component
  if(TRUE) {
    e <- e_orig/sqrt(lT1@g)
    
    #lG1 <- MTV1@garch                # For first iteration
    lG1 <- MTV1$results[[1]]$garch   # For subsequent iterations - update the index!
    summary(lG1)
    ll.target <- getTargetValue(e,garchObj = lG1)
    lG1$pars <- lG1$Estimated$pars
    
    lG1$pars["omega",1] <- 0.02
    lG1$pars["alpha",1] <- 0.05
    lG1$pars["beta",1] <- 0.80
    lG1$pars["gamma",1] <- 0.05
    
    optCtrl <- lG1$optimcontrol
    optCtrl$reltol <- 1e-9
    optCtrl$ndeps <- c(1e-7,1e-7,1e-7,1e-7)
    optCtrl$parscale <- c(1,1,9,1)
    lG1$optimcontrol <- optCtrl
    
    lG1 <- estimateGARCH(e,lG1,estCtrl)
    summary(lG1)  
    plot(lG1)
    
    # Check that the LL is higher than the target:
    lG1$Estimated$value > ll.target 
    
  }
  
  # Now call the calculate liklihood method for mtvgarch
  
  MTV1 <- calcLoglik(MTV1,lT1,lG1)
  
  # Pick the best specification & write it to the Estimated List:
  MTV1$Estimated[[1]] <- MTV1$results[[2]]
  
  
  # Save the estimated model
  saveRDS(MTV1,"Output/CBA_mtvgjr_manual.RDS")
}

##==============================================================##
##                            THE END
##==============================================================##

MTV1 <- readRDS("Output/CBA_mtvgjr_manual.RDS")
summary(MTV1$results[[1]]$tv)
plot(MTV1$results[[1]]$tv)
summary(MTV1$results[[2]]$tv)
plot(MTV1$results[[2]]$tv)



