rm(list=ls())
gc(TRUE)

##====================== Initialisation ========================####
if (TRUE){
  setwd("C:/GoogleDrive/MyDocs/Annastiina/Topics/MTVGJR_MGARCH/Project2_Building")        # Anna's work PC  - GOOGLE DRIVE
  #setwd("~/Project2_Building")                                         
  
  source("clsTV.r")
  source("clsGARCH.r")
  source("clsMTVGARCH.r")
  library(parallel)
  library(RevoUtilsMath)
  maxThreads <- detectCores()-1
  setMKLthreads(maxThreads)
}
##====================== Initialisation ========================##


##====================== Data Load =============================####
if (TRUE){
  # Read AUS_4Banks data from saved file
  mydata <- readRDS("AUS_bankreturns_1992_2020.RDS")
  dates <- mydata$date
  
  # Create scaled-up & de-meaned individual data series vectors (Percentage Returns):
  e_wbc <- mydata$return_wbc * 100 
  e_wbc <- e_wbc - mean(e_wbc)
  e <- e_wbc
  
  # Plot the returns
  ptitle <- "WBC Returns"
  plot(dates,e,type="l",main=ptitle)
  
  # Read in the Garch'd Reference Data
  RefData <- readRDS("Gen_Refdata/RefData_withGarch_WBC_2020.RDS")
  
  # Tidy Up
  rm(mydata)
  gc(TRUE)
}
##====================== Data Load =============================##


##   
##============  General Specification Method    ===============####
##
##   1. Plot the local Variance and visually observe any transitions
##   2. Calculate a set of *indicative* garch parameters using different window lengths & kurtosis
##   3. Identify the first "single" transition start & end:
##     4. Generate refData0 using TV$delta0 = var(e[start:end])
##     5. Test for a first transition in this subset of the time series 
##     6. Generate refData1 using TV$delta0 & TV$pars = TV estimates found above
##     7. Test for a second transition in this subset of the time series, and if none is found, move on to the next subset 
##   8. Repeat steps 4 - 7 for each subsequent data subset where we think there may be a transition
##   9. Complete the specification by adding the individual sub-sections together, re-estimating & re-testing
##  10. Consider if any of the transitions could be re-parameterised as a double-transition to reduce the total number of parameters
##  11. Re-estimate the final model and calculate & check the standard errors

##==============================================================##


# Plot returns using index as x-axis:
ptitle <- "WBC Returns"
plot(e,type="l",main = ptitle)
abline(h=seq(-10,10,5), v=seq(0,7000,500),col="lightgrey")
lines(e)



##==============================================================##
##  TV.ORDER 0, obs 1:7108  ####
##==============================================================##

if (TRUE) {

  e <- e_wbc
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  TV <- tv(st,tvshape$delta0only)
  TV <- estimateTV(e,TV)
  
  # Now Test against WBC
  TV <- setTaylorOrder(1,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "WBC.ORD0.1_1-7108.RDS"
  simcontrol$numLoops <- 1000
  simcontrol$numCores <- 3
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)

  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}  
##==============================================================##
##  TV - AUS_4Banks/WBC ---- ORDER 0.1 ---- obs 1:7108          ##

## Result: Pvals=[TR2: 73%,Robust= 69%] 
## Fail to Reject the null: H0=Model is sufficient
##  This is likely caused by issues with the optimiser 
##  where there are many transitions
##  So, try a smaller range... 
##==============================================================##



##==============================================================##
##  TV.ORDER 0, obs 1:4000  ####
##==============================================================##

if (TRUE) {
  
  e <- e_wbc[1:4000]
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  TV <- tv(st,tvshape$delta0only)
  TV <- estimateTV(e,TV)
  
  # Now Test against WBC
  TV <- setTaylorOrder(2,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "WBC.ORD0.2_1-4000.RDS"
  simcontrol$numLoops <- 1200
  simcontrol$numCores <- 3
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)
  
  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}  # End: if (DoThis)...
##==============================================================##
##  TV - AUS_4Banks/WBC ---- obs 1:4000          

## ORDER 0.1 
## Result: Pvals=[TR2= 3%,Robust= 4%] 
## ORDER 0.2
## Result: Pvals=[TR2= 8%,Robust= 2%] 
## ORDER 0.3
## Result: Pvals=[TR2= %,Robust= %] 

## Reject the null: H0=Model is sufficient
## Conclusion: Strong evidence of a transition here

##==============================================================##


##==============================================================##
##  TV.ORDER 1, obs 1:4000  ####
##==============================================================##

if (TRUE) {
  
  e <- e_wbc[1:4000]
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  # Add some estimation controls
  estCtrl <- list()
  estCtrl$calcSE <- TRUE
  estCtrl$verbose <- TRUE
  
  TV <- tv(st,tvshape$single)
  
  OptCtrl <- TV$optimcontrol
  OptCtrl$reltol <- 1e-9
  OptCtrl$ndeps <- c(1e-7,1e-7,1e-7,1e-5)
  #OptCtrl$parscale <- c(6,6,3,1)
  TV$optimcontrol <- OptCtrl
  
  TV$delta0 <- 0.5
  TV$pars[1,1] <- 0.01
  TV$pars[2,1] <- 2.05
  TV$pars[3,1] <- 0.6

  TV <- estimateTV(e,TV,estCtrl)
  
  summary(TV)
  plot(TV)
  
  # Now Test against WBC
  TV <- setTaylorOrder(2,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "WBC.ORD1.2_1-4000.RDS"
  simcontrol$numLoops <- 1200
  simcontrol$numCores <- 3
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)
  
  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}
##==============================================================##

# Estimation Results:
#   
#   Delta0 = 0.000207 se0 =  7e-06 
# 
#             st1      se1
# deltaN -0.000113 0.000009
# speedN  2.875682 0.276979
# locN1   0.646473 0.029049
# locN2         NA      NaN
# 
# Log-liklihood value:  11811.23

##==============================================================##

##  TV - AUS_4Banks/WBC ---- ---- obs 1:5000          
## ORDER 1.1 
## Result: Pvals=[TR2= 92%,Robust= 90%] 
## ORDER 1.2
## Result: Pvals=[TR2= 40%,Robust= 43%] 
## ORDER 1.3
## Result: Pvals=[TR2= %,Robust=%] 

## Cannot Reject the null
## Conclusion: No evidence of another transition here

##==============================================================##



##==============================================================##
##  TV.ORDER 0, obs 3501:5500  ####
##==============================================================##

if (TRUE) {
  
  e <- e_wbc[3501:5500]
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  TV <- tv(st,tvshape$delta0only)
  
  # Add some estimation controls
  estCtrl <- list()
  estCtrl$calcSE <- TRUE
  estCtrl$verbose <- TRUE
  
  OptCtrl <- TV$optimcontrol
  OptCtrl$reltol <- 1e-11
  OptCtrl$ndeps <- OptCtrl$ndeps * 1e-2
  #OptCtrl$parscale
  TV$optimcontrol <- OptCtrl
  
  TV$delta0
  TV$pars[1,1] <- 0.05
  TV$pars[2,1] <- 2.05
  TV$pars[3,1] <- 0.65
  
  TV <- estimateTV(e,TV,estCtrl)
  
  summary(TV)
  plot(TV)
  
  # Now Test against WBC
  TV <- setTaylorOrder(2,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "WBC.ORD0.2_3501-5500.RDS"
  simcontrol$numLoops <- 1200
  simcontrol$numCores <- 3
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)
  
  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}  # End: if (DoThis)...
##==============================================================##
##  TV - AUS_4Banks/WBC ---- ---- obs 3501:5500
## ORDER 0.1 
## Result: Pvals=[TR2= 52%,Robust= 23%] 
## ORDER 0.2
## Result: Pvals=[TR2= 3.4%,Robust= 0.4%] 
## ORDER 0.3
## Result: Pvals=[TR2= %,Robust= %] 

## Reject the null
## Conclusion: Strong evidence of a double transition here
##             Next: Lets try to estimate the 2 transitions we've found
##==============================================================##



##==============================================================##
##  TV.ORDER 2, obs 1:5500  ####
##==============================================================##

if (TRUE) {
  
  e <- e_wbc[1:5500]
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  # Add some estimation controls
  estCtrl <- list()
  estCtrl$calcSE <- TRUE
  estCtrl$verbose <- TRUE
  
  TV <- tv(st,c(tvshape$single,tvshape$double1loc))

  OptCtrl <- TV$optimcontrol
  OptCtrl$reltol <- 1e-9
  OptCtrl$ndeps <- c(1e-7,1e-7,1e-5,1e-5,1e-7,1e-7,1e-5)
  #OptCtrl$parscale <- c(6,6,2,1,6,2,1)
  OptCtrl$parscale <- c(3,3,1,1,3,1,1)
  TV$optimcontrol <- OptCtrl
  
  TV$delta0 <- 0.1
  TV$pars["deltaN",1] <- 0.05
  TV$pars["speedN",1] <- 3.05
  TV$pars["locN1",1] <- 0.3
  TV$pars["deltaN",2] <- 0.05
  TV$pars["speedN",2] <- 3.05
  TV$pars["locN1",2] <- 0.6
  
  TV <- estimateTV(e,TV,estCtrl)
  
  summary(TV)
  plot(TV)
  
  # Now Test against WBC
  TV <- setTaylorOrder(2,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "WBC.ORD2sd.2_1-5500.RDS"
  simcontrol$numLoops <- 1200
  simcontrol$numCores <- 3
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)
  
  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}

##==============================================================##

# Estimation Results:
#   
#   Delta0 = 0.001825 se0 =  0.000163 
# 
#             st1       st2      se1      se2
# deltaN -0.000072 -0.001614 0.000008 0.000163
# speedN  3.461076  6.999988 0.418388 0.175481
# locN1   0.397173  0.774516 0.022339 0.003241
# locN2         NA        NA      NaN      NaN
# 
# Log-liklihood value:  15732.82 

##==============================================================##

##==============================================================##
##  TV - AUS_4Banks/WBC ---- ---- obs 1:5500          
## ORDER 2.1 
## Result: Pvals=[TR2= 32%,Robust= 16%] 
## ORDER 2.2
## Result: Pvals=[TR2= 39%,Robust= 33%] 

## Fail to Reject the null
## Conclusion: No Evidence of another transition here

##==============================================================##



##==============================================================##
##  TV.ORDER 2, obs 1:7108  ####
##==============================================================##

if (TRUE) {
  
  e <- e_wbc[1:7108]
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  # Add some estimation controls
  estCtrl <- list()
  estCtrl$calcSE <- TRUE
  estCtrl$verbose <- TRUE
  
  TV <- tv(st,c(tvshape$single,tvshape$double1loc))
  
  OptCtrl <- TV$optimcontrol
  OptCtrl$reltol <- 1e-9
  OptCtrl$ndeps <- c(1e-7,1e-7,1e-5,1e-5,1e-7,1e-7,1e-5)
  OptCtrl$parscale <- c(6,6,1,1,6,1,1)
  #OptCtrl$parscale <- c(3,3,1,1,3,1,1)
  TV$optimcontrol <- OptCtrl
  
  TV$delta0 <- 0.1
  TV$pars["deltaN",1] <- 0.05
  TV$pars["speedN",1] <- 3.05
  TV$pars["locN1",1] <- 0.3
  TV$pars["deltaN",2] <- 0.05
  TV$pars["speedN",2] <- 3.05
  TV$pars["locN1",2] <- 0.6
  
  TV <- estimateTV(e,TV,estCtrl)
  
  summary(TV)
  plot(TV)
  
  # Now Test against WBC
  TV <- setTaylorOrder(2,TV)
  
  RefTests <- list()
  RefTests$LMTR2 <- LM.TR2(e,TV)
  RefTests$LMRobust <- LM.Robust(e,TV)
  
  simcontrol <- list()
  simcontrol$saveAs <- "WBC.ORD2sd.2_1-7108.RDS"
  simcontrol$numLoops <- 1200
  simcontrol$numCores <- 4
  TEST <- testStatDist(TV,RefData,RefTests,simcontrol)
  
  # View Distributions
  hist(TEST$Stat_TR2,breaks = 15)
  hist(TEST$Stat_Robust,breaks = 15)
  
}

##==============================================================##

# Estimation Results:
#   
#   Delta0 = 0.001618 se0 =  0.000105 
# 
#             st1       st2      se1      se2
# deltaN -0.000079 -0.001407 0.000008 0.000104
# speedN  2.963292  7.000000 0.252549 0.000000
# locN1   0.319891  0.606221 0.019991 0.001652
# locN2         NA        NA      NaN      NaN
# 
# Log-liklihood value:  20590.96

##==============================================================##

##==============================================================##
##  TV - AUS_4Banks/WBC ---- ---- obs 1:7108          
## ORDER 2.1 
## Result: Pvals=[TR2= 47%,Robust= 93%] 
## ORDER 2.2
## Result: Pvals=[TR2= NaN%,Robust= 99.8%] 

## Fail to Reject the null
## Conclusion: No Evidence of another transition here
##             Model is fully specified
##==============================================================##


##==============================================================##
##  TV.ORDER 2, MTVGJR Model  ####
##==============================================================##

##====================== Data Load =============================####
if (TRUE){
  # Read AUS_4Banks data from saved file
  mydata <- readRDS("AUS_bankreturns_1992_2020.RDS")
  dates <- mydata$date
  
  # Create scaled-up & de-meaned individual data series vectors (Percentage Returns):
  e_wbc <- mydata$return_wbc * 100 
  e_wbc <- e_wbc - mean(e_wbc)
  e <- e_wbc
  
  # Plot the returns
  ptitle <- "WBC Returns"
  plot(dates,e,type="l",main=ptitle)
  
  # Read in the Garch'd Reference Data
  RefData <- readRDS("Gen_Refdata/RefData_withGarch_WBC_2020.RDS")
  
  # Tidy Up
  rm(mydata)
  gc(TRUE)
}

if (TRUE) {
  
  source("clsMTVGARCH.r")
  
  e <- e_wbc
  nr.obs <- length(e)
  st <- 1:nr.obs/nr.obs
  
  TV <- tv(st,c(tvshape$single,tvshape$double1loc))
  
  OptCtrl <- TV$optimcontrol
  OptCtrl$reltol <- 1e-9
  OptCtrl$ndeps <- c(1e-7,1e-7,1e-5,1e-5,1e-7,1e-7,1e-5)
  OptCtrl$parscale <- c(6,6,1,1,6,1,1)
  TV$optimcontrol <- OptCtrl
  
  TV$delta0 <- 1
  TV$pars["deltaN",1] <- 1
  TV$pars["speedN",1] <- 3.05
  TV$pars["locN1",1] <- 0.3
  TV$pars["deltaN",2] <- 1
  TV$pars["speedN",2] <- 3.05
  TV$pars["locN1",2] <- 0.6
  
  estCtrl <- list(calcSE = TRUE,verbose = TRUE)
  TV <- estimateTV(e,TV,estCtrl)
  
  summary(TV)
  plot(TV)

  # Create the MTV-GJR object
  MTV_wbc <- mtvgarch(TV,garchType = garchtype$gjr)
  
  # Estimate the MTV-GJR object
  MTV_wbc <- estimateMTVGARCH(e,MTV_wbc)  
  
  # Look at the univariate MTV-GJR object:
  # Plot univar TV 'g'
  plot(MTV_wbc$tv)
  # Overlay the final estimated TV 'g'
  lines(MTV_wbc$Estimated$tv@g,type="l",col="red")
  # If the scale is off, plot the Estimated TV 'g'
  plot(MTV_wbc$Estimated$tv@g,type="l",col="red")
  
  # Plot univar Garch 'h'
  plot(MTV_wbc$garch@h,type="l")
  # Overlay the final estimated Garch 'h'
  lines(MTV_wbc$Estimated$garch@h,type="l",col="red")
  # If the scale is off, plot the Estimated Garch 'h'
  plot(MTV_wbc$Estimated$garch@h,type="l",col="red")
  
  # Look at the final parameters
  summary(MTV_wbc$Estimated$tv)
  summary(MTV_wbc$Estimated$garch)
  
  # Save the estimated model
  saveRDS(MTV_wbc,"Output/WBC_mtvgjr.RDS")
  

}


## Recommend the Order 3 model



##==============================================================##
##  MTVGJR Model - Manual Estimation   ####
##==============================================================##

if (TRUE) {
  
  ##==============================================================##
  ##  TV - AUS_4Banks\WBC ---- ORDER 3 ---- obs 1:7027            ##
  ##
  ## Result: Pvals=[TR2: %, Robust: %], using
  ## TestStat_ProbDist_WBC Logliklihood Value:  ??

  
  # Create e, TV1 & G1 
  # make an original copy of the data:
  e_orig <- e
  estCtrl <- list(calcSE = TRUE, verbose = TRUE)
  st <- 1:length(e)/length(e)
  
  TV1 <- tv(st,c(tvshape$single,tvshape$single,tvshape$single))
  #startpars <- c(1,3,0.4,5,3,0.5,-5,3,0.7)
  #parScale <- c(1,1,1,0.3,3,1,0.3,3,1,0.3)
  
  TV1$delta0 <- 1
  #TV1$pars["deltaN",] <- c(-1.0,8.0,-7.0)   # -12063.84
  #TV1$pars["deltaN",] <- c(-1.0,12.0,-10.0)   # -12067.78
  #TV1$pars["deltaN",] <- c(-1.0,15.0,-12.0)   # -12067.78
  TV1$pars["deltaN",] <- c(-1.0,20.0,-15.0)   # -12067.78
  #TV1$pars["speedN",] <- c(3.0,3.0,3.0)
  TV1$pars["speedN",] <- c(2.0,3.0,3.0) #-12057.45
  TV1$pars["locN1",] <- c(0.5,0.6,0.7)
  
  optCtrl <- TV1$optimcontrol
  optCtrl$reltol <- 1e-9
  optCtrl$ndeps <- c(1e-5, 1e-7,1e-7,1e-5, 1e-5,1e-5,1e-5, 1e-5,1e-5,1e-5)
  optCtrl$parscale <- c(1, 2,1,1, 10,3,1, 10,3,1)
  TV1$optimcontrol <- optCtrl
  
  TV1 <- estimateTV(e,TV1,estCtrl)
  summary(TV1)  
  plot(TV1)
  
  
  # TV OBJECT
  # 
  # Estimation Results:
  #   
  #   Delta0 = 2.446516 se0 =  0.324652 
  # 
  # st1      se1 sig       st2      se2 sig.1        st3      se3 sig.2
  # deltaN -3.531029 0.705131 *   16.621524 5.527461   *   -14.631516 5.502481   *  
  #   speedN  1.690180 0.292989 *    4.699384 0.141237   **    5.523306 0.422883   *  
  #   locN1   0.543742 0.029114 *    0.582660 0.005750   **    0.608307 0.002483   ***
  #   locN2         NA      NaN            NA      NaN               NA      NaN      
  # 
  # Log-liklihood value:  -12057.45
  
  
  G1 <- garch(garchtype$gjr)
  
  OptCtrl <- G1$optimcontrol
  OptCtrl$reltol <- 1e-7
  OptCtrl$ndeps <- c(1e-5,1e-5,1e-5,1e-5)
  OptCtrl$parscale <- c(1,1,9,1)
  G1$optimcontrol <- OptCtrl
  
  G1$pars["omega",1] <- 0.01
  G1$pars["alpha",1] <- 0.06
  G1$pars["beta",1] <- 0.8
  G1$pars["gamma",1] <- 0.03
  
  G1 <- estimateGARCH(e,G1,estCtrl)
  G1 <- estimateGARCH(e/sqrt(TV1@g),G1,estCtrl)
  summary(G1)
  plot(G1)
  
  MTV1 <- mtvgarch(e,TV1,G1)
  
  ###  START THE PAIR_WISE ESTIMATION HERE  ###
  
  # Filter out Garch & estimate TV component
  if(TRUE) {
    
    lG1 <- MTV1@garch                # For first iteration
    lG1 <- MTV1$results[[1]]$garch   # For subsequent iterations - update the index!
    e <- e_orig/sqrt(lG1@h)
    
    lT1 <- MTV1@tv                # For first iteration
    lT1 <- MTV1$results[[1]]$tv   # For subsequent iterations - update the index!
    
    summary(lT1)
    ll.target <- getTargetValue(e,lT1)
    
    lT1$delta0 <- lT1$Estimated$delta0
    lT1$pars <- lT1$Estimated$pars 
    
    lT1$pars["deltaN",] <-   c(-3.0,10.0,-9.0)#c(-1.0,20.0,-15.0) #c(-1.0,9.0,-8.0)
    lT1$pars["speedN",] <- c(2.0,4.0,4.0)
    lT1$pars["locN1",] <- c(0.4,0.6,0.7)
    #lT1$pars["locN2",] <- 0
    
    optCtrl <- lT1$optimcontrol
    optCtrl$reltol <- 1e-9
    optCtrl$ndeps <- c(1e-7,1e-5,1e-5,1e-5,1e-5,1e-5,1e-5,1e-5,1e-5)
    optCtrl$parscale <-   c(3,1,1,30,1,1,30,1,1) #c(3,1,1,5,1,1,5,1,1) #c(3,1,1,3,1,1,3,1,1)
    lT1$optimcontrol <- optCtrl
    
    lT1 <- estimateTV(e,lT1,estCtrl)
    summary(lT1)  
    plot(lT1)
    
    # Check that the LL is higher than the target:
    lT1$Estimated$value > ll.target 
    
  }
  
  # Filter out TV & estimate GARCH component
  if(TRUE) {
    e <- e_orig/sqrt(lT1@g)
    
    lG1 <- MTV1@garch                # For first iteration
    lG1 <- MTV1$results[[1]]$garch   # For subsequent iterations - update the index!
    summary(lG1)
    ll.target <- getTargetValue(e,garchObj = lG1)
    lG1$pars <- lG1$Estimated$pars
    
    lG1$pars["omega",1] <- 0.10
    lG1$pars["alpha",1] <- 0.05
    lG1$pars["beta",1] <- 0.80
    lG1$pars["gamma",1] <- 0.02
    
    optCtrl <- lG1$optimcontrol
    optCtrl$reltol <- 1e-9
    optCtrl$ndeps <- c(1e-7,1e-7,1e-7,1e-7)
    optCtrl$parscale <- c(1,1,9,1)
    lG1$optimcontrol <- optCtrl
    
    lG1 <- estimateGARCH(e,lG1,estCtrl)
    summary(lG1)  
    plot(lG1)
    
    # Check that the LL is higher than the target:
    lG1$Estimated$value > ll.target 
    
  }
  
  # Now call the calculate liklihood method for mtvgarch
  
  MTV1 <- calcLoglik(MTV1,lT1,lG1)
  
  # Pick the best specification & write it to the Estimated List:
  MTV1$Estimated[[1]] <- MTV1$results[[1]]
  
  
  # Save the estimated model
  saveRDS(MTV1,"Output/WBC_mtvgjr_manual.RDS")
}

##==============================================================##
##                            THE END
##==============================================================##
