rm(list=ls())
gc(TRUE)

##====================== Initialisation ========================####
if (TRUE){
  setwd("C:/GoogleDrive/MyDocs/Annastiina/Topics/MTVGJR_MGARCH/Project2_Building")        # Anna's work PC  - GOOGLE DRIVE
  #setwd("~/Project2_Building")                                         
  
  ## Note:  
  ##  This file was re-written after the univariate TV-Specification work was completed
  ##  The earlier clsTV, clsGARCH, etc. files have now been compiled into the MTVGARCH package
  ##  A method is provided on the Garch class 'estimateGARCH_RollingWindow()' to simplify & ecapsulate this task
  
  library(MTVGARCH)

}
##====================== Initialisation ========================##



##====================== Data Load =============================####
if (TRUE){
  # Read AUS_4Banks data from saved file
  mydata <- readRDS("AUS_bankreturns_1992_2020.RDS")
  dates <- mydata$date

  # Create scaled-up & de-meaned individual data series vectors (Percentage Returns):
  e_anz <- (mydata$return_anz - mean(mydata$return_anz)) * 100
  e_cba <- (mydata$return_cba - mean(mydata$return_cba)) * 100
  e_nab <- (mydata$return_nab - mean(mydata$return_nab)) * 100
  e_wbc <- (mydata$return_wbc - mean(mydata$return_wbc)) * 100

  # Plot the returns
  ptitle <- "ANZ Returns"
  plot(dates,e_anz,type="l",main=ptitle)
  plot(e_anz,type="l",main=ptitle)
  abline(v=seq(1,7000,200),col="grey")
  ## Stable variance period: 2004 - 2007 (OBS: 2900/3000 - 4000)

  ptitle <- "CBA Returns"
  plot(dates,e_cba,type="l",main=ptitle)
  plot(e_cba,type="l",main=ptitle)
  abline(v=seq(1,7000,200),col="grey")
  ## Stable variance period: 2004 - 2007 (OBS: 400-1400, 3000 - 4000)

  ptitle <- "NAB Returns"
  plot(dates,e_nab,type="l",main=ptitle)
  plot(e_nab,type="l",main=ptitle)
  abline(v=seq(1,7000,200),col="grey")
  ## Stable variance period: 2004 - 2007 (OBS: 200-1400)

  ptitle <- "WBC Returns"
  plot(dates,e_wbc,type="l",main=ptitle)
  plot(e_wbc,type="l",main=ptitle)
  abline(v=seq(1,7000,200),col="grey")
  ## Stable variance period: 2004 - 2007 (OBS: 2900 - 3900)


}
##================ END: Data Load ================##


##================ Method 1: Analyse a Constant/Stable Variance Period for each Bank ================####

estControl <- list(calcSE=TRUE)

G_anz <- garch(garchtype$general)
G_anz <- estimateGARCH(e_anz,G_anz,estControl)
plot(G_anz,main="Garch - ANZ")
summary(G_anz)
G_anz_sub <- estimateGARCH(e_anz[3000:4000],G_anz,estControl)
summary(G_anz_sub)
gar <- G_anz_sub
per <- round(gar$Estimated$pars["alpha",1] + gar$Estimated$pars["beta",1],3)
kur <- round(3*(1-(per)^2)/(1-(per)^2-2*gar$Estimated$pars["alpha",1]^2),3)
#3*(1-(a+b)^2)/(1-(a+b)^2-2*a^2)


G_cba <- garch(garchtype$general)
G_cba <- estimateGARCH(e_cba,G_cba,estControl)
plot(G_cba,main="Garch - CBA")
summary(G_cba)
G_cba_sub <- estimateGARCH(e_cba[3000:4000],G_cba,estControl)
summary(G_cba_sub)
gar <- G_cba_sub
per <- round(gar$Estimated$pars["alpha",1] + gar$Estimated$pars["beta",1],3)
kur <- round(3*(1-(per)^2)/(1-(per)^2-2*gar$Estimated$pars["alpha",1]^2),3)

G_nab <- garch(garchtype$general)
G_nab <- estimateGARCH(e_nab,G_nab,estControl)
plot(G_nab,main="Garch - NAB")
summary(G_nab)
G_nab_sub <- estimateGARCH(e_nab[3000:4000],G_nab,estControl)
summary(G_nab_sub)
gar <- G_nab_sub
per <- round(gar$Estimated$pars["alpha",1] + gar$Estimated$pars["beta",1],3)
kur <- round(3*(1-(per)^2)/(1-(per)^2-2*gar$Estimated$pars["alpha",1]^2),3)

G_wbc <- garch(garchtype$general)
G_wbc <- estimateGARCH(e_wbc,G_wbc,estControl)
plot(G_wbc,main="Garch - WBC")
summary(G_wbc)
G_wbc_sub <- estimateGARCH(e_wbc[3000:4000],G_wbc,estControl)
summary(G_wbc_sub)
gar <- G_wbc_sub
per <- round(gar$Estimated$pars["alpha",1] + gar$Estimated$pars["beta",1],3)
kur <- round(3*(1-(per)^2)/(1-(per)^2-2*gar$Estimated$pars["alpha",1]^2),3)


InitialGarchGuess <- list()

InitialGarchGuess$ANZ <- G_anz
InitialGarchGuess$ANZ_sub <- G_anz_sub
InitialGarchGuess$CBA <- G_cba
InitialGarchGuess$CBA_sub <- G_cba_sub
InitialGarchGuess$NAB <- G_nab
InitialGarchGuess$NAB_sub <- G_nab_sub
InitialGarchGuess$WBC <- G_wbc
InitialGarchGuess$WBC_sub <- G_wbc_sub

saveRDS(InitialGarchGuess,"InitialGarchGuess.RDS")

##================ END: Constant/Stable Variance Period ================##


##================ Method 2: Rolling Window Method ================####


# omega = 1 - (a - b) * local variance  (Implemented in the Garch Class)

estControl <- list(verbose=TRUE, calcSE=FALSE, vartargetWindow=400)

##====  Change the data, and process each bank (one at a time) ===##
e <- e_wbc

G2 <- garch(garchtype$general)
G2 <- estimateGARCH_RollingWindow(e,G2,estControl)

summary(G2)
gar <- G2
per <- round(gar$Estimated$pars["alpha",1] + gar$Estimated$pars["beta",1],3)
kur <- round(3*(1-(per)^2)/(1-(per)^2-2*gar$Estimated$pars["alpha",1]^2),3)


##================ END: Rolling Window Method ================##


