rm(list=ls())
Sys.sleep(2)
gc(TRUE)
   
Sys.sleep(2)
# Clear the console
cat("\f") 
timestamp()
##====================== Initialisation ========================####
if (TRUE){
  setwd("C:/GoogleDrive/MyDocs/Annastiina/Topics/MTVGJR_MGARCH/Project2_Building")        # Anna's work PC  - GOOGLE DRIVE
  #setwd("~/Project2_Building")                                         
  
  ## Note:  
  ##  This file was re-written after the univariate TV-Specification work was completed
  ##  The earlier clsTV, clsGARCH, etc. files have now been compiled into the MTVGARCH package
  ##  A method is provided on the Garch class 'estimateGARCH_RollingWindow()' to simplify & ecapsulate this task
  
  library(MTVGARCH)

}
##====================== Initialisation ========================##

##================ Rolling Window Method - Simulation ================####

# omega = 1 - (a - b) * local variance  (Implemented in the Garch Class::estimateGARCH_RollingWindow())


#    Generate RefData with desired TV & Garch characteristics
#    Specifically, simulate the following:
#
#    In all cases use the same Garch spec: alpha = 0.1, beta = 0.85

#    1. Data with no TV component (const variance)  - RefData: Data1
#    2. Data with TV = 1 transition, shape = 1      - RefData: Data2
#    3. Data with TV = 1 transition, shape = 2      - RefData: Data3
#    4. Data with TV = 2 transitions, shape = (1,1) - RefData: Data4
#    5. Data with TV = 2 transitions, shape = (1,1) - RefData: Data5


##=============== 1. Generate Reference Data =================####
if(FALSE){
  
# Clear the console
cat("\f")  
  
# nr.obs:
Obs <- 5000
St <- 1:Obs/Obs
e <- rnorm(Obs)
# nr.series:
N <- 1000

# GARCH:
G1 <- garch(garchtype$general)
G1$pars["alpha",1] <- 0.10
G1$pars["beta",1] <- 0.85

# TV:
Shape <- c(tvshape$delta0only)
T1 <- tv(St,Shape)
Data1 <- generateRefData(T1,G1,nr.series = N,Obs)
saveRDS(Data1,"Gen_Refdata/refData_TV_Const_RollWindow_Garch_Sim.RDS")

Sys.sleep(2)

Shape <- c(tvshape$single)
T1 <- tv(St,Shape)
T1$delta0 <- 1.0
T1$pars[1,1] <- 3.0
T1$pars[2,1] <- 3.0
T1$pars[3,1] <- 0.5
Data2 <- generateRefData(T1,G1,nr.series = N,Obs)
saveRDS(Data2,"Gen_Refdata/refData_TV_1Trans_RollWindow_Garch_Sim.RDS")

Sys.sleep(2)

Shape <- c(tvshape$single,tvshape$single)
T1 <- tv(St,Shape)
T1$delta0 <- 1.0
T1$pars[1,] <- c(3.0,-3)
T1$pars[2,] <- c(3.5,3.5)
T1$pars[3,] <- c(0.33,0.66)
Data3 <- generateRefData(T1,G1,nr.series = N,Obs)
saveRDS(Data3,"Gen_Refdata/refData_TV_2Trans_Symmetric_RollWindow_Garch_Sim.RDS")

Sys.sleep(2)

Shape <- c(tvshape$single,tvshape$single)
T1 <- tv(St,Shape)
T1$delta0 <- 1.0
T1$pars[1,] <- c(8.0,-5)
T1$pars[2,] <- c(3.0,6.0)
T1$pars[3,] <- c(0.33,0.66)
Data4 <- generateRefData(T1,G1,nr.series = N,Obs)
saveRDS(Data4,"Gen_Refdata/refData_TV_2Trans_Exciting_RollWindow_Garch_Sim.RDS")

Sys.sleep(2)

Shape <- c(tvshape$single,tvshape$single,tvshape$single,tvshape$single)
T1 <- tv(St,Shape)
T1$delta0 <- 1.0
T1$pars[1,] <- c(3.0,-3,8,-8)
T1$pars[2,] <- c(4,4,4.5,5)
T1$pars[3,] <- c(0.25,0.35,0.65,0.75)
Data5 <- generateRefData(T1,G1,nr.series = N,Obs)
saveRDS(Data5,"Gen_Refdata/refData_TV_4Trans_Exotic_RollWindow_Garch_Sim.RDS")

cat("Game Over for the refData generation")

# Quick test & view:
e1 <- Data3[,5]
plot(e1,type='l')

T2 <- .calculate_g(T1)
plot(T2,type='l')

}  # End: generate Ref Data Sets

##=============== 2. Run simulation, 1 data set at a time =================####

# 2. Run a simulation using the same GARCH, with 4 different Window sizes:
#    for each of the different types of TV-transition, TV: Data1..Data5

if(TRUE){

library(doParallel)
nr.cores <- 7
cl <- makeCluster(nr.cores)
Sys.sleep(2)
registerDoParallel(cl)
Sys.sleep(2)

# GARCH:
G1 <- garch(garchtype$general)
G1$pars["alpha",1] <- 0.10
G1$pars["beta",1] <- 0.85
estCtrl <- list(verbose=FALSE, calcSE=FALSE, vartargetWindow=1)

# TV: Data1..Data5
# e <- readRDS("Gen_Refdata/refData_TV_Const_RollWindow_Garch_Sim.RDS")
# fileName <- "Output/TV_Const_RollWindow_Garch_Sim.RDS"
# #
# e <- readRDS("Gen_Refdata/refData_TV_1Trans_RollWindow_Garch_Sim.RDS")
# fileName <- "Output/TV_1Trans_RollWindow_Garch_Sim.RDS"
#
# e <- readRDS("Gen_Refdata/refData_TV_2Trans_Symmetric_RollWindow_Garch_Sim.RDS")
# fileName <- "Output/TV_2Trans_Symmetric_RollWindow_Garch_Sim.RDS"
#
# e <- readRDS("Gen_Refdata/refData_TV_2Trans_Exciting_RollWindow_Garch_Sim.RDS")
# fileName <- "Output/TV_2Trans_Exciting_RollWindow_Garch_Sim.RDS"
# #
e <- readRDS("Gen_Refdata/refData_TV_4Trans_Exotic_RollWindow_Garch_Sim.RDS")
fileName <- "Output/TV_4Trans_Exotic_RollWindow_Garch_Sim_detail.RDS"

N <- NCOL(e)
Obs <- NROW(e)
dummyCols <- matrix(NA,nrow = N,ncol = 2)  #Use to add persistence & kurtosis to resultset

simResults <- list()
#vtWinLen <- c(250,500,750,1000)
## For the 4-trans, exotic case, lets look in more detail:
vtWinLen <- c(300,350,400,450)

for(n in 1:4){
  estCtrl$vartargetWindow <- vtWinLen[n]
  
  simResult <- foreach(i = 1:N, .combine = 'rbind', .inorder = FALSE, .export = "estimateGARCH_RollingWindow") %dopar% {
    G1 <- estimateGARCH_RollingWindow(e[,i],G1,estCtrl)
    #Return
    c(vtWinLen[n], as.vector(G1$Estimated$pars))
  }
  Sys.sleep(1)
  simResult <- cbind(simResult,dummyCols)
  colnames(simResult) <- c("winLen","omega","alpha","beta","pers","kurt")
  
  simResult[,"pers"] <- simResult[,"alpha"] + simResult[,"beta"]
  simResult[,"kurt"] <- 3*(1-(simResult[,"alpha"]+simResult[,"beta"])^2)/(1-(simResult[,"alpha"]+simResult[,"beta"])^2-2*simResult[,"alpha"]^2)
  
  simResults[[n]] <- simResult
  saveRDS(simResults,fileName)  # Save & overwrite after each winLen is processed
  Sys.sleep(2)
  
}

## 

stopCluster(cl)
cat("Simulation is done!")

}


## ===== Analysis  =======####

if(FALSE){
  
  # Step 1:  Pick your dataset
  fileName <- "Output/TV_Const_RollWindow_Garch_Sim.RDS"
  fileName <- "Output/TV_1Trans_RollWindow_Garch_Sim.RDS"
  fileName <- "Output/TV_2Trans_Symmetric_RollWindow_Garch_Sim.RDS"
  fileName <- "Output/TV_2Trans_Exciting_RollWindow_Garch_Sim.RDS"
  fileName <- "Output/TV_4Trans_Exotic_RollWindow_Garch_Sim.RDS"
  fileName <- "Output/TV_4Trans_Exotic_RollWindow_Garch_Sim_detail.RDS"
  simResults <- readRDS(fileName)
  
  # Step 2:  Pick your Window Length (1-4 = [250,500,750,1000])
  # Window Length for xxx_detail.RDS: (1-4 = [300,350,400,450])
  ResultIdx <- 3
  
  # Step 3:  Look at the histograms
  
  #a
  hist(simResults[[ResultIdx]][,"alpha"],breaks = 30)
  abline(v=0.1,lty=2)
  
  #b
  hist(simResults[[ResultIdx]][,"beta"],breaks = 30)
  abline(v=0.85,lty=2)
  
  #a+b
  hist(simResults[[ResultIdx]][,"pers"],breaks = 30)
  abline(v=0.95,lty=2)
  
  #3*(1-(a+b)^2)/(1-(a+b)^2-2*a^2)
  hist(simResults[[ResultIdx]][,"kurt"],breaks = 30)
  abline(v=3*(1-(0.1+0.85)^2)/(1-(0.1+0.85)^2-2*0.1^2),lty=2)

}

timestamp()

## ===== End  =======####







