rm(list=ls())
gc(T)

####====================== Initialisation ========================####
if (TRUE){

  setwd("C:/GoogleDrive/MyDocs/Annastiina/Topics/MTVGJR_MGARCH/Project2_Building")        # Anna's work PC  - GOOGLE DRIVE
  #setwd("~/Project2_Building") 
  
  library(ggplot2)
  library(MTVGARCH)
  # Set the graph window dimensions for landscape, font size 16:
  win.graph(11,8,16)

}
##====================== Initialisation ========================##

##====================== Data Load =============================####
if (TRUE){
  # Read AUS_4Banks data from saved file
  mydata <- readRDS("AUS_bankreturns_1992_2020.RDS")
  dates <- mydata$date

  e_anz <- 100 * ( mydata$return_anz - mean(mydata$return_anz) )
  e_cba <- 100 * ( mydata$return_cba - mean(mydata$return_cba) )
  e_nab <- 100 * ( mydata$return_nab - mean(mydata$return_nab) )
  e_wbc <- 100 * ( mydata$return_wbc - mean(mydata$return_wbc) )

  tvgANZ <- readRDS("Output/tvgjr_ANZ.RDS")
  tvgCBA <- readRDS("Output/tvgjr_CBA.RDS")
  tvgNAB <- readRDS("Output/tvgjr_NAB.RDS")
  tvgWBC <- readRDS("Output/tvgjr_WBC.RDS")

}
##====================== Data Load =============================##


####====================== Plot the Returns =========================####
DoThis <- TRUE
if (DoThis) {
  win.graph(14,8,26)
  par(mfrow=c(1,1),mar=c(bottom=4,left=2,top=2,right=2))

  e <- cbind(e_anz,e_cba,e_nab,e_wbc)

  Title <- c("ANZ")
  d <- data.frame(dates,e[,1])

  Title <- c("CBA")
  d <- data.frame(dates,e[,2])

  Title <- c("NAB")
  d <- data.frame(dates,e[,3])

  Title <- c("WBC")
  d <- data.frame(dates,e[,4])

  ##  Plot Variables:
  fontSize <- 26  #

  # Colour Plot
  lineSize <- 0.5
  # B/W Plot
  lineSize <- 0.5

  # Now Plot Returns:
  colnames(d) <- c("Time","Returns")
  ggplot(d) +
    geom_line(aes(x=Time, y=Returns),colour = "grey10", size=lineSize) +
    xlab('') + ylab('') + theme_bw() +
    ggtitle(Title) + theme(plot.title= element_text(size = (fontSize+2), hjust = 0.5,margin=margin(b=10) ),
                           axis.text = element_text(size = fontSize) ) +
    scale_y_continuous(breaks = c(-10,-5,0,5,10),limits = c(-12,12))


  # Now Plot Abs(Returns) + 'g':

  e <- abs(cbind(e_anz,e_cba,e_nab,e_wbc))
  g <- sqrt(cbind(tvgANZ$Estimated$tv@g,tvgCBA$Estimated$tv@g,tvgNAB$Estimated$tv@g,tvgWBC$Estimated$tv@g))

  Title <- c("ANZ")
  d <- data.frame(dates,e[,1],g[,1])

  Title <- c("CBA")
  d <- data.frame(dates,e[,2],g[,2])

  Title <- c("NAB")
  d <- data.frame(dates,e[,3],g[,3])

  Title <- c("WBC")
  d <- data.frame(dates,e[,4],g[,4])

  colnames(d) <- c("Time","Returns","g")
  ggplot(d) +
    geom_line(aes(x=Time, y=Returns),colour = "grey80", size=lineSize) +
    geom_line(aes(x=Time, y=g), size=lineSize+0.5,colour = "grey10") +
    xlab('') + ylab('') + theme_bw() +
    ggtitle(Title) + theme(plot.title= element_text(size = fontSize+2, hjust = 0.5, margin=margin(b=10)),
                                                    axis.text = element_text(size = fontSize)
                                                      ) +
    scale_y_continuous(breaks = c(2,4,6,8,10),limits = c(0,12))

}

####====================== End: Plot the Returns =========================##


####====================== Plot the TV'g' over Results =========================####

DoThis <- TRUE
if (DoThis) {

  win.graph(12,8,16)
  
  par(mfrow=c(2,2),mar=c(bottom=4,left=5,top=3,right=2))  # Display All on 1 page
  #par(mfrow=c(1,1),mar=c(bottom=4,left=5,top=3,right=2))  # Display just 1 per page

  g <- sqrt(tvgANZ$Estimated$tv@g)
  ptitle <- "ANZ"
  plot(dates,g,"l",main = "",cex.axis=1.7,xlab="",ylab="",lwd=2,panel.first = grid(lty=1,col = "cornsilk2"))  # Dates on x-axis
  mtext("ANZ",side = 3,line = 0,cex=2)

  g <- sqrt(tvgCBA$Estimated$tv@g)
  ptitle <- "CBA"
  plot(dates,g,"l",main = "",cex.axis=1.7,xlab="",ylab="",lwd=2,panel.first = grid(lty=1,col = "cornsilk2"))  # Dates on x-axis
  mtext("CBA",side = 3,line = 0,cex=2)

  g <- sqrt(tvgNAB$Estimated$tv@g)
  ptitle <- "NAB"
  plot(dates,g,"l",main = "",cex.axis=1.7,xlab="",ylab="",lwd=2,panel.first = grid(lty=1,col = "cornsilk2"))  # Dates on x-axis
  mtext("NAB",side = 3,line = 0,cex=2)

  g <- sqrt(tvgWBC$Estimated$tv@g)
  ptitle <- "WBC"
  plot(dates,g,"l",main = "",cex.axis=1.7,xlab="",ylab="",lwd=2,panel.first = grid(lty=1,col = "cornsilk2"))  # Dates on x-axis
  mtext("WBC",side = 3,line = 0,cex=2)

}

####====================== Plot the Auto-Correlations =========================####
DoThis <- TRUE
if (DoThis) {

  win.graph(24,16,12)
  par(mfrow=c(4,1),mar=c(bottom=3,left=8,top=2,right=2))

  titleFontsize <- 1.2
  ymax <- 0.3
  lags <- 500
  startlag <- 1

  banks <- c("ANZ","CBA","NAB","WBC")
  bankRtns <- cbind(e_anz,e_cba,e_nab,e_wbc)


  for (n in seq_along(banks)){
    # AC of Returns^2
    ret2 <- ts(bankRtns[,n]^2,start = 1,end=7108,frequency = 1)
    acf1 <- acf(ret2,lag.max = lags,type = "corr",plot = FALSE)
    # Plot without the first result (which always = 1.0)
    plot(acf1,type="h",main = "",cex.axis=1.5,lwd=2,xlab="",xlim=c(startlag,lags),ylim = c(-0.05,ymax),
         ylab="",panel.first = grid(lty=1,col = "cornsilk2"))   # Index on x-axis
    mtext(banks[n],side = 2, cex=titleFontsize, las=2,line = 3)
  }
  
  ## Plot filtered Auto-Correlations 1 at a time:
  par(mfrow=c(1,1),mar=c(bottom=3,left=3,top=2,right=2))
  
  # Autocorrelation of demeaned % returns:
  e <- e_wbc
  ret2 <- ts(e^2,start = 1,end=7108,frequency = 1)
  acf1 <- acf(ret2,lag.max = lags,type = "corr",plot = FALSE)
  # Plot without the first result (which always = 1.0)
  plot(acf1,type="h",main = "",cex.axis=1.5,lwd=2,xlab="",xlim=c(startlag,lags),ylim = c(-0.05,ymax),
       ylab="",panel.first = grid(lty=1,col = "cornsilk2"))   # Index on x-axis
  mtext("WBC Returns",side = 3, cex=titleFontsize, las=1,line = 0) #Title on Top
  #mtext("WBC Returns",side = 2, cex=titleFontsize, las=2,line = 5) #Title on LHS

  # Autocorrelation of demeaned % returns filtered by 'g':
  e <- e_wbc/sqrt(tvgWBC$Estimated$tv@g)
  ret2 <- ts(e^2,start = 1,end=7108,frequency = 1)
  acf1 <- acf(ret2,lag.max = lags,type = "corr",plot = FALSE)
  # Plot without the first result (which always = 1.0)
  plot(acf1,type="h",main = "",cex.axis=1.5,lwd=2,xlab="",xlim=c(startlag,lags),ylim = c(-0.05,ymax),
       ylab="",panel.first = grid(lty=1,col = "cornsilk2"))   # Index on x-axis
  mtext("WBC Returns/sqt(g)",side = 3, cex=titleFontsize, las=1,line = 0) #Title on Top

  e <- e_wbc/sqrt(tvgWBC$Estimated$garch@h)
  ret2 <- ts(e^2,start = 1,end=7108,frequency = 1)
  acf1 <- acf(ret2,lag.max = lags,type = "corr",plot = FALSE)
  # Plot without the first result (which always = 1.0)
  plot(acf1,type="h",main = "",cex.axis=1.5,lwd=2,xlab="",xlim=c(startlag,lags),ylim = c(-0.05,ymax),
       ylab="",panel.first = grid(lty=1,col = "cornsilk2"))   # Index on x-axis
  mtext("WBC Returns/sqt(h)",side = 3, cex=titleFontsize, las=1,line = 0) #Title on Top

  TV_g <- sqrt(tvgWBC$Estimated$tv@g)
  GARCH_h <- sqrt(tvgWBC$Estimated$garch@h)
  e <- e_wbc/sqrt((TV_g*GARCH_h))
  ret2 <- ts(e^2,start = 1,end=7108,frequency = 1)
  acf1 <- acf(ret2,lag.max = lags,type = "corr",plot = FALSE)
  # Plot without the first result (which always = 1.0)
  plot(acf1,type="h",main = "",cex.axis=1.5,lwd=2,xlab="",xlim=c(startlag,lags),ylim = c(-0.05,ymax),
       ylab="",panel.first = grid(lty=1,col = "cornsilk2"))   # Index on x-axis
  mtext("WBC Returns/sqt(g.h)",side = 3, cex=titleFontsize, las=1,line = 0) #Title on Top


}


####  ========  Plot NAB with TV & Garch for slides  ======== ####
DoThis <- TRUE
if (DoThis) {

e <- ts(e_nab,start = c(1992,1),frequency = 252)
g <- sqrt(tvgNAB$Estimated$tv@g)
h <- sqrt(tvgNAB$Estimated$garch@h)

win.graph(10,8,14)
par(mfcol=c(2,1),mar=c(bottom=3,left=5,top=2,right=2))

plot.ts(e,type = 'l',xlab="",ylab="",ylim = c(-15,15),panel.first = grid())   #lty=1,col = "cornsilk2"))
mtext("  Returns",side=3,line=-2,cex=1.25)

plot(h+g-mean(h),type = 'l',xlab="",ylab="", panel.first = grid(lty=1,col = "cornsilk2"))
lines(g,type = 'l',col="red",lwd=3)
#plot(h+g-mean(h),type = 'l',ylim = c(2.5,5.5),xlab="",ylab="",lwd=1,panel.first = grid(lty=1,col = "cornsilk2"))

#mtext("  TV & Garch",side=3,line=-2,cex=1.25)
}


##  When does the Corr Transition start & end?
DoThis <- TRUE
if (DoThis){

  ## -- USING G(t) -- ##

  Gt <- myG

  plot(Gt,type='l')
  pc05 <- 0.05*(max(Gt) - min(Gt))

  lev_s <- min(Gt) + pc05/2
  lev_e <- max(Gt) - pc05/2

  tmp <- Gt[Gt < lev_s]
  loc_s <- length(tmp)
  mydata[loc_s,1]
  # Start is at loc[2735] = 2002-10-16

  tmp <- Gt[Gt < lev_e]
  loc_e <- length(tmp)
  mydata[loc_e,1]
  # Start is at loc[5273] = 2012-10-31

  Gt.ts <- ts(Gt,start=c(1992,1),frequency = 252)
  plot(Gt.ts,typ='l',lwd=2,panel.first = grid(nx=61,ny=5,lty=1,col="cornsilk2"))
  abline(h=c(lev_s,lev_e),col="blue",lty=4)
  t_c <- time(Gt.ts)
  abline(v=c(t_c[loc_s],t_c[loc_e]),col="blue",lty=3)

  ## Corr transition: End of Oct-2002 to mid Oct-2012
}


####  =======   Plot the correlations  ====== ####
DoThis <- TRUE
if (DoThis){
STCC1 <- readRDS("Output/AUS_4Banks_STCC_20201028.RDS")

  library(ggplot2)
  library(reshape2)

  # CoVariance Plot - for Timo
  if(T){

  corAll <- STCC1$Estimated$Pt
  cov1 <- corAll[,1] * sqrt(tvgANZ$Estimated$tv@g) * sqrt(tvgANZ$Estimated$garch@h) * sqrt(tvgCBA$Estimated$tv@g) * sqrt(tvgCBA$Estimated$garch@h)
  cov2 <- corAll[,2] * sqrt(tvgANZ$Estimated$tv@g) * sqrt(tvgANZ$Estimated$garch@h) * sqrt(tvgNAB$Estimated$tv@g) * sqrt(tvgNAB$Estimated$garch@h)
  cov3 <- corAll[,3] * sqrt(tvgANZ$Estimated$tv@g) * sqrt(tvgANZ$Estimated$garch@h) * sqrt(tvgWBC$Estimated$tv@g) * sqrt(tvgWBC$Estimated$garch@h)
  cov4 <- corAll[,4] * sqrt(tvgCBA$Estimated$tv@g) * sqrt(tvgCBA$Estimated$garch@h) * sqrt(tvgNAB$Estimated$tv@g) * sqrt(tvgNAB$Estimated$garch@h)
  cov5 <- corAll[,5] * sqrt(tvgCBA$Estimated$tv@g) * sqrt(tvgCBA$Estimated$garch@h) * sqrt(tvgWBC$Estimated$tv@g) * sqrt(tvgWBC$Estimated$garch@h)
  cov6 <- corAll[,6] * sqrt(tvgNAB$Estimated$tv@g) * sqrt(tvgNAB$Estimated$garch@h) * sqrt(tvgWBC$Estimated$tv@g) * sqrt(tvgWBC$Estimated$garch@h)

  CovMat <- matrix(NA,7108,6)
  CovMat[,6] <- cov6
  STCC1$Estimated$Covariance <- CovMat

  covv <- data.frame(cbind(mydata[,1],CovMat))
  colnames(covv) <- c("date","ANZ:CBA","ANZ:NAB","ANZ:WBC","CBA:NAB","CBA:WBC","NAB:WBC")
  d1 <- melt(covv, id="date")
  colnames(d1) <- c("date","Covariance","value")

  ggplot(d1, aes(x=date, y=value, color=Covariance)) +
    geom_line(aes(linetype=Covariance), size=1) +
    xlab("") + ylab("")+ theme_grey()

  }


  ####====  Generate data to plot Correlations with line-markers every 300 obs ====####
  m1 <- matrix(NA,7108,6)
  for(n in seq(1,7108,by=300)){
    if(n < 7108) m1[(n),] <- STCC1$Estimated$Pt[n,]
  }
  plot(m1[,1])

  corr1 <- data.frame(cbind(mydata[,1],STCC1$Estimated$Pt[,1]))
  colnames(corr1) <- c("date","corr")
  corr2 <- data.frame(cbind(mydata[,1],STCC1$Estimated$Pt[,2]))
  colnames(corr2) <- c("date","corr")
  corr3 <- data.frame(cbind(mydata[,1],STCC1$Estimated$Pt[,3]))
  colnames(corr3) <- c("date","corr")
  corr4 <- data.frame(cbind(mydata[,1],STCC1$Estimated$Pt[,4]))
  colnames(corr4) <- c("date","corr")
  corr5 <- data.frame(cbind(mydata[,1],STCC1$Estimated$Pt[,5]))
  colnames(corr5) <- c("date","corr")
  corr6 <- data.frame(cbind(mydata[,1],STCC1$Estimated$Pt[,6]))
  colnames(corr6) <- c("date","corr")


  corr <- data.frame(cbind(mydata[,1],m1))
  colnames(corr) <- c("Time","ANZ:CBA","ANZ:NAB","ANZ:WBC","CBA:NAB","CBA:WBC","NAB:WBC")
  d <- melt(corr, id="Time")
  colnames(d) <- c("Time","Correlations","value")


  ## Print to window:
  par(mfcol=c(1,1),mar=c(bottom=3,left=5,top=2,right=2))

  ##  Plot Variables:
  shapes <- c(5,1,2,15,16,17)
  trans <- c(2735,5273)  # Start & End of transition
  fontSize <- 18  #
  shapeSize <- 2.5


  # Colour Plot
  cols <- c("ANZ:CBA" = "blue","ANZ:NAB" = "grey30","ANZ:WBC" = "orange","CBA:NAB" = "yellow2","CBA:WBC" = "grey80","NAB:WBC" = "purple")
  lineSize <- 0.75
  # B/W Plot
  cols <- c("ANZ:CBA" = "grey10","ANZ:NAB" = "grey10","ANZ:WBC" = "grey10","CBA:NAB" = "grey10","CBA:WBC" = "grey10","NAB:WBC" = "grey10")
  lineSize <- 0.50

  # Now Plot:
  ggplot(d, aes(x=Time, y=value)) +
    geom_point(aes(shape=Correlations,colour = Correlations), size=shapeSize) +
    scale_shape_manual(values=shapes) +
    geom_line(data = corr1, aes(x = date, y = corr),colour = cols[1], size=lineSize) +
    geom_line(data = corr2, aes(x = date, y = corr),colour = cols[2], size=lineSize) +
    geom_line(data = corr3, aes(x = date, y = corr),colour = cols[3], size=lineSize) +
    geom_line(data = corr4, aes(x = date, y = corr),colour = cols[4], size=lineSize) +
    geom_line(data = corr5, aes(x = date, y = corr),colour = cols[5], size=lineSize) +
    geom_line(data = corr6, aes(x = date, y = corr),colour = cols[6], size=lineSize) +
    geom_vline(xintercept=as.numeric(d$Time[trans]),colour = "grey40", linetype = "dashed" ) +
    scale_colour_manual(values = cols) +
    xlab('') + ylab('') + theme_bw() +
    theme(text = element_text(size=fontSize),legend.justification=c(1,0), legend.position=c(0.945,0.05), legend.background = element_rect(colour="grey80", size=1.5))


  ## Show available colours
  colours()

  ## Show a list of available shapes
  df_shapes <- data.frame(shape = 0:24)
  ggplot(df_shapes, aes(0, 0, shape = shape)) +
    geom_point(aes(shape = shape), size = 5, fill = 'red') +
    scale_shape_identity() +
    facet_wrap(~shape) +
    theme_bw()



## Basic R Plots:

win.graph(18,14,12)
par(mfcol=c(1,1),mar=c(bottom=3,left=5,top=2,right=2))

corr <- ts(STCC1$Estimated$Pt,start=c(1992,1),frequency = 252)
t_c <- time(corr)

legendText <- c("ANZ:CBA","ANZ:NAB","ANZ:WBC","CBA:NAB","CBA:WBC","NAB:WBC")

# COLOUR PLOT:
plot(corr,plot.type="single",xlab="",ylab="",lwd=3.5,
     lty=c("solid","longdash","dotted","solid","longdash","dotted"),
     col=c("yellow","green","red","darkgrey","blue","black"),
     panel.first = grid(nx=53,ny=5,lty=1,col="cornsilk2")
     )
abline(v=c(t_c[loc_s],t_c[loc_e]),col="blue",lty="dotdash")

legend(x="topleft",inset=0.03,legendText,bg="White",fill=c("yellow","green","red","darkgrey","blue","black"),ncol=3)

}

####======  Plot 'g(t)' for STCC  ======####

stcc <- STCC1

gInv <- 1+exp(-exp(stcc$Estimated$pars[1])*(stcc@st-stcc$Estimated$pars[2]))
g <- 1/gInv

ptitle <- "STCC"
plot(dates,g,"l",main = "",cex.axis=1.7,xlab="",ylab="",lwd=2,panel.first = grid(lty=1,col = "cornsilk2"))  # Dates on x-axis
mtext("STCC1",side = 3,line = 0,cex=2)

##======  End: Plot 'g(t)' for STCC  ======##


