
# Clear the Environment
try(rm(list=ls()))

## Source the tv class
source("clsGARCH.r")

#Create some dummy data
set.seed(1999)
myData <- rnorm(1000,0,2)
plot(myData,type='l')

#### Create a GARCH object, type = NoGarch ####
if(TRUE){
  myG0 <- garch(garchtype$noGarch)
  
  # View the GARCH object - note the slots (internal variables) are not visible
  View(myG0)
  
  # To see everything (including the slots), use the print() method
  print(myG0)
  
  # Estimate this GARCH object:
  myG0 <- estimateGARCH(myData,myG0)
  ## Not valid, so Should throw an error
  
}


#### Create a GARCH object, type = General ####
if(TRUE){
  myG1 <- garch(garchtype$general)
  # View the GARCH object - note the slots (internal variables) are not visible
  View(myG1)
  # To see everything (including the slots), use the print() method
  print(myG1)
  # Estimate this GARCH object:
  myG1 <- estimateGARCH(myData,myG1)
  
  myG1$pars
  myG1$Estimated$pars
  
  # Now change the starting parameters & set some controls before estimating:
  myG1$pars[1,1] <- 0.20
  myG1$pars[2,1] <- 0.05
  myG1$pars[3,1] <- 0.90
  
  # More explicitly
  myG1$pars["omega",1] <- 0.20
  myG1$pars["alpha",1] <- 0.05
  myG1$pars["beta",1] <- 0.90
  
  estCtrl <- list()
  estCtrl$calcSE <- TRUE
  estCtrl$verbose <- TRUE
  
  # Estimate this GARCH object:
  myG1 <- estimateGARCH(myData,myG1,estCtrl)
  View(myG1)
  # Plot - default = cond. variance
  plot(myG1)
  
  # View the estimation results
  summary(myG1)
  
}


#### Control the optimiser for Estimation ####
if(TRUE){
  myG1 <- garch(garchtype$general)
  
  # Update the starting params
  myG1$pars["omega",1] <- 0.20
  myG1$pars["alpha",1] <- 0.05
  myG1$pars["beta",1] <- 0.90
  
  # OPTIONAL: Set the estimation controls
  estCtrl <- list()
  estCtrl$calcSE <- TRUE
  estCtrl$verbose <- TRUE
  
  # OPTIONAL: Set the Optimiser Controls
  OptCtrl <- list(fnscale = -1, maxit = 1500, reltol = 1e-7, ndeps = c(1e-4,1e-4,1e-5), parscale = c(1,1,5))
  myG1$optimcontrol <- OptCtrl
    
  # Set any one control:
  myG1$optimcontrol$fnscale <- -100
  
  # Estimate this GARCH object:
  myG1 <- estimateGARCH(myData,myG1,estCtrl)
  
  # View the estimation results
  summary(myG1)
  
}

#### TEST: All Garch Types ####
if(TRUE){
  
  #Create some dummy data
  set.seed(1999)
  myData <- rnorm(1000,0,2)
  
  ## NO GARCH
  myG0 <- garch(garchtype$noGarch)
  # Estimate this GARCH object: Not valid, so should throw an error
  myG0 <- estimateGARCH(myData,myG0)
  # View the Estimated params: Should be NULL
  myG0$Estimated$pars
  # Plot the conditional variance: Not valid, so should throw an error
  plot(myG0)
  # View the estimation results
  summary(myG0)
  
  
  ## GENERAL
  myG1 <- garch(garchtype$general)
  # Estimate this GARCH object:
  myG1 <- estimateGARCH(myData,myG1,list(calcSE=T))
  # View the Estimated params:
  myG1$Estimated$pars
  # Plot the conditional variance: 
  plot(myG1)
  # View the estimation results
  summary(myG1)
  
  ## GJR
  myG2 <- garch(garchtype$gjr)
  # Estimate this GARCH object:
  myG2 <- estimateGARCH(myData,myG2,list(calcSE=T))
  # View the Estimated params:
  myG2$Estimated$pars
  # Plot the conditional variance: 
  plot(myG2)
  # View the estimation results
  summary(myG2)

  # Remove the Test objects
  rm(myG1,myG2,myData)
}
