
# Clear the Environment
try(rm(list=ls()))


## Source the tv class
source("MTVGJR/clsTV.r")

#Create dummy data
set.seed(1999)
myData <- rnorm(1000,0,2)
plot(myData,type='l')

#Create a linear smooth transition
st <- (1:1000)/1000

#### Create a TV object, with DELTA0 Only ####
if(TRUE){
  myShape <- tvshape$delta0only
  myTV0 <- tv(st,myShape)
  
  # View the TV object - note the slots (internal variables) are not visible
  View(myTV0)
  
  # To see everything (including the slots), use the print() method
  print(myTV0)
  
  # Estimate this TV object:
  myTV0 <- estimateTV(myData,myTV0)
  
  ## Note: We did not provide the optional estimationControl list parameter for this simple case
  
  # Now View the estimated TV object:
  View(myTV0)

}

#### Create a TV object, with ONE single-shaped transition ####
if(TRUE){
  myShape <- tvshape$single
  myTV1 <- tv(st,myShape)
  View(myTV1)
  myTV1 <- estimateTV(myData,myTV1)
  View(myTV1)
  
  # Now change the starting parameters & set some controls before estimating:
  
  myTV1$pars
  myTV1$delta0 <- 1
  myTV1$pars[1,1] <- 1
  myTV1$pars[2,1] <- 2
  myTV1$pars[3,1] <- 0.2
  
  # More explicitly
  myTV1$pars["deltaN",1] <- 1
  myTV1$pars["speedN",1] <- 2
  myTV1$pars["locN1",1] <- 0.2
  
  estCtrl <- list()
  estCtrl$calcSE <- TRUE
  estCtrl$verbose <- TRUE
  
  myTV1 <- estimateTV(myData,myTV1,estCtrl)
  View(myTV1)
  
  # Plot the conditional variance
  plot(myTV1)
  
  # View the estimation results
  summary(myTV1)
  
}

#### Create a TV object, with TWO transitions ####
if(TRUE){
  myShape <- c(tvshape$single,tvshape$double)
  myTV2 <- tv(st,myShape)
  myTV2 <- estimateTV(myData,myTV2)
  View(myTV2)
  
  # Now change the starting parameters & set some controls before estimating:
  myTV2$pars
  myTV2$delta0 <- 3
  myTV2$pars[1,1] <- 1
  myTV2$pars[2,1] <- 2
  myTV2$pars[3,1] <- 0.3
  #
  myTV2$pars[1,2] <- -1
  myTV2$pars[2,2] <- 3.5
  myTV2$pars[3,2] <- 0.7
  myTV2$pars[4,2] <- 0.8
  
  # OPTIONAL: Set the estimation controls
  estCtrl <- list()
  estCtrl$calcSE <- TRUE
  estCtrl$verbose <- TRUE
  
  # OPTIONAL: Set the Optimiser Controls
  OptCtrl <- list(fnscale = -1, maxit = 500, reltol = 1e-11)
  myTV2$optimcontrol <- OptCtrl
  
  # Set any one control:
  myTV2$optimcontrol$fnscale <- -100
  
  myTV2 <- estimateTV(myData,myTV2,estCtrl)
  View(myTV2)
  
  # PLot
  plot(myTV2)
  
  # View the estimation results
  summary(myTV2)
  
}

#### Test a TV model for evidence of more transitions ####
if(TRUE){
  
  # Notes on Taylor Order: 
  # Order 1 => approximate a linear transition (1 param = maximum power)
  # Order 2 => approximate a quadratic transition (2 params = reduced power)
  # Order 3 => approximate a cubic transition (3 params = further reduced power)
  # Order 4 => approximate a complex transition (4 params = very low power)
  
  #Create dummy data & linear transition
  set.seed(1999)
  myData <- rnorm(1000,0,2)
  st <- (1:1000)/1000
  
  # 1. Create & Estimate a TV Object
  myShape <- tvshape$single
  myTV1 <- tv(st,myShape)
  View(myTV1)
  myTV1 <- estimateTV(myData,myTV1)
  plot(myTV1)
  
  # 2. Set the Taylor Expansion Order on an estimated TV object:
  myTV1 <- setTaylorOrder(1,myTV1)   
  
  # 3. Test for evidence of more transitions
  TR2.Test <- LM.TR2(myData,myTV1)
  Robust.Test <- LM.Robust(myData,myTV1)
  
  # Order = 2
  myTV1 <- setTaylorOrder(2,myTV1)   
  TR2.Test <- LM.TR2(myData,myTV1)
  Robust.Test <- LM.Robust(myData,myTV1)
  
  # Order = 3
  myTV1 <- setTaylorOrder(3,myTV1)   
  TR2.Test <- LM.TR2(myData,myTV1)
  Robust.Test <- LM.Robust(myData,myTV1)
  
  # Order = 4
  myTV1 <- setTaylorOrder(4,myTV1)   
  TR2.Test <- LM.TR2(myData,myTV1)
  Robust.Test <- LM.Robust(myData,myTV1)
  
}

#### Generate a Test-Statistic Distribution for a given model-specification ####
if(TRUE){
  
  
  
}


